@@ -528,7 +528,17 @@ class SignalInstance:
         >>> t.sig.emit(5)
         >>> assert my_obj.x == 5
         """
-        ref = obj if isinstance(obj, weakref.ref) else weakref.ref(obj)
+        try:
+            ref = obj if isinstance(obj, weakref.ref) else weakref.ref(obj)
+        except TypeError:  # pragma: no cover
+            import warnings
+
+            warnings.warn(
+                f"Could not create weakref to `{obj}`. Call `disconnect_setattr` "
+                "manually upon object deletion to avoid memory leaks.",
+                RuntimeWarning,
+            )
+            ref = lambda: obj  # type: ignore [assignment] # noqa: E731
         if not hasattr(ref(), attr):
             raise AttributeError(f"Object {ref()} has no attribute {attr!r}")
 
@@ -623,13 +633,17 @@ class SignalInstance:
         >>> t.sig.emit(5)
         >>> assert my_obj == {'x': 5}
         """
-        if isinstance(obj, weakref.ref):
-            ref = obj  # pragma: no cover
-        else:
-            try:
-                ref = weakref.ref(obj)
-            except TypeError:
-                ref = lambda: obj  # type: ignore # noqa # object doesn't support weakref
+        try:
+            ref = obj if isinstance(obj, weakref.ref) else weakref.ref(obj)
+        except TypeError:
+            import warnings
+
+            warnings.warn(
+                f"Could not create weakref to `{obj}`. Call `disconnect_setitem` "
+                "manually upon object deletion to avoid memory leaks.",
+                RuntimeWarning,
+            )
+            ref = lambda: obj  # type: ignore [assignment] # noqa: E731
 
         if not hasattr(ref(), "__setitem__"):
             raise TypeError(f"Object {ref()} does not support __setitem__")
@@ -747,6 +761,10 @@ class SignalInstance:
             idx = self._slot_index(slot)
             if idx != -1:
                 self._slots.pop(idx)
+                if isinstance(slot, PartialMethod):
+                    _PARTIAL_CACHE.pop(id(slot), None)
+                elif isinstance(slot, tuple) and callable(slot[2]):
+                    _prune_partial_cache()
             elif not missing_ok:
                 raise ValueError(f"slot is not connected: {slot}")
 
@@ -1237,16 +1255,33 @@ def _is_subclass(left: Type[Any], right: type) -> bool:
     return issubclass(left, right)
 
 
+_PARTIAL_CACHE: dict[int, tuple[weakref.ref, str, Callable]] = {}
+
+
 def _partial_weakref(slot_partial: PartialMethod) -> tuple[weakref.ref, str, Callable]:
     """For partial methods, make the weakref point to the wrapped object."""
-    ref, name = _get_method_name(slot_partial.func)
-    args_ = slot_partial.args
-    kwargs_ = slot_partial.keywords
+    _id = id(slot_partial)
 
-    def wrap(*args: Any, **kwargs: Any) -> Any:
-        getattr(ref(), name)(*args_, *args, **kwargs_, **kwargs)
+    # if the exact same partial is used twice, we don't want to recreate a new
+    # wrap() function, because we want _partial_weakref(cb) == _partial_weakref(cb)
+    # to be True.  So we cache the result of the first call using the id of the partial
+    if _id not in _PARTIAL_CACHE:
+        ref, name = _get_method_name(slot_partial.func)
+        args_ = slot_partial.args
+        kwargs_ = slot_partial.keywords
 
-    return (ref, name, wrap)
+        def wrap(*args: Any, **kwargs: Any) -> Any:
+            getattr(ref(), name)(*args_, *args, **kwargs_, **kwargs)
+
+        _PARTIAL_CACHE[_id] = (ref, name, wrap)
+    return _PARTIAL_CACHE[_id]
+
+
+def _prune_partial_cache() -> None:
+    """Remove any partial methods whose object has been garbage collected."""
+    for key, (ref, *_) in list(_PARTIAL_CACHE.items()):
+        if ref() is None:
+            del _PARTIAL_CACHE[key]
 
 
 def _get_method_name(slot: MethodType) -> tuple[weakref.ref, str]:
